/*
* This file is part of fwmon
* Copyright (c) 2001,2002 Gianni Tedesco
*
*    fwmon is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    fwmon is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with fwmon; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

static const char rcsid[]=
	"@(#) $Id: netlink.c,v 1.11 2002/02/04 08:35:03 gianni Exp $";

#include "g_globals.h"
#include "netlink.h"
#include "main.h"
#include <errno.h>
#include <sys/time.h>

#ifdef NETLINK_USE_SOCKET
#include <asm/types.h>
#include <linux/netlink.h>
struct sockaddr_nl nls;
#endif

int netlink_fd=-1;

#ifndef NETLINK_USE_SOCKET

/*
 * Using a character device (depricated, i believe)
 */
int netlink_open(void)
{
	if ( (netlink_fd=open( NETLINK_DEV, O_RDONLY ))<0 ) 
		return 0;
	return 1;
}

int netlink_read(void *data, int len)
{
	return read( netlink_fd, data, len );
}

#else

/* 
 * Using a proper netlink socket
 */

int netlink_open(void)
{
	if ( (netlink_fd=socket(PF_NETLINK, SOCK_RAW, NETLINK_FIREWALL))<0 )
		return 0;

	memset( &nls, 0, sizeof(nls));
	nls.nl_family = AF_NETLINK;
	nls.nl_groups = -1;

	if ( bind( netlink_fd, (struct sockaddr *)&nls, sizeof(nls)) )
		return 0;

	return 1;
}

int netlink_read(void *data, int len)
{
	return recv( netlink_fd, data, len, 0 );
}

#endif

int netlink_close(void)
{
	return close(netlink_fd);
}

void netlink_mainloop(void)
{
	int len;
	struct data_t *dptr;
	packet_t packet;

	if ( !(dptr=malloc(sizeof(struct data_t))) ) {
		perror("allocating packet buffer");
		return;
	}
	
	memset(&packet, 0, sizeof(packet));
	packet.hdr=(struct netlink_t *)&dptr->netlink;
	packet.ip=(struct iphdr *)&dptr->data;
	
	for(;;) {
		len = netlink_read( dptr, sizeof(*dptr));

		if ( len <= 0 ) {
			if ( !flag(FLAG_DROPROOT) ) {
				/* eek netlink died, reopen it */
				if ( len < 0 ) perror("read()");
				sleep(1);
				netlink_close();
				netlink_open(); /* best effort */
				continue;
			}else{
				print_err(ERR_RESYNC CRLF);
				break;
			}
		}

		if ( len < (sizeof(*dptr) - BUFFER_SIZE) ){
			print_err(ERR_SMALL_PKT CRLF);
		}else{
			/* If the kernel sends a netlink packet bigger
			* than our read buffer size */
			if ( packet.hdr->len > BUFFER_SIZE ) {
				packet.hdr->len=BUFFER_SIZE;
			}
		
			packet.hdr->len-=sizeof(struct netlink_t);
			gettimeofday(&packet.ts, NULL);
			dump_packet(&packet);
		}
	}

	free(dptr);
}
