/*
* This file is part of fwmon
* Copyright (c) 2001,2002 Gianni Tedesco
*
*    fwmon is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    fwmon is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with fwmon; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

static const char rcsid[]=
	"@(#) $Id: slog.c,v 1.6 2002/01/31 09:38:37 gianni Exp $";

#include "g_globals.h"
#include "slog.h"
#include "util.h"
#include <syslog.h>

static char buf[BUFFER_SIZE];

int slog_init(void)
{
	if ( flag(FLAG_SYSLOG) ) {
		openlog("fwmon", LOG_CONS|LOG_PID|LOG_NDELAY, LOG_DAEMON);
	}
	return 1;
}

void slog_close(void)
{
	if ( flag(FLAG_SYSLOG) ) {
		closelog();
	}
}

void slog_dump ( p_packet p )
{
	static char src_ip[32], dst_ip[32];
	struct tcphdr  *tcp = p->trans;
	struct udphdr  *udp = p->trans;
	struct icmphdr *icmp = p->trans;
	int sport=0, dport=0;
	char *b=buf;

	b[0]='\0';
	format_ip(p->ip->saddr, src_ip);
	format_ip(p->ip->daddr, dst_ip);
	
	/* set ports, dependant on protocol */
	switch ( p->ip->protocol ){
		case IPPROTO_TCP:
			sport=ntohs(tcp->source);
			dport=ntohs(tcp->dest);
			break;
		case IPPROTO_UDP:
			sport=ntohs(udp->source);
			dport=ntohs(udp->dest);
			break;
		case IPPROTO_ICMP:
			sport=icmp->type;
			dport=icmp->code;
			break;
	}

	/* Log interface and IP header stuff */
	b+=sprintf(b, "IN=%s SRC=%s DST=%s PROTO=%s LEN=%i TTL=%i",
		p->hdr->iface,
		src_ip,
		dst_ip,
		get_proto(p->ip->protocol),
		ntohs(p->ip->tot_len),
		p->ip->ttl);
	
	switch ( p->ip->protocol ) {
		case IPPROTO_TCP:
			b+=sprintf(b, " TCP_FLAGS=%s", get_tcpflags(tcp));
		case IPPROTO_UDP:
			b+=sprintf(b, " DPT=%i SPT=%i", dport, sport);
			break;
		case IPPROTO_ICMP:
			b+=sprintf(b, " ICMP_TYPE=%i ICMP_CODE=%i ICMP_NAME=%s",
				sport,
				dport,
				icmp_type(sport, dport));
			break;
	}

	syslog(LOG_WARNING, (char *)&buf);
}
